package com.loafle.overflow.central.module.infra.service;

import com.loafle.overflow.central.commons.utils.PageUtil;
import com.loafle.overflow.central.module.infra.dao.InfraDAO;
import com.loafle.overflow.central.module.infra.dao.InfraHostDAO;
import com.loafle.overflow.central.module.infra.dao.InfraServiceDAO;
import com.loafle.overflow.central.module.infra.dao.InfraZoneDAO;
import com.loafle.overflow.central.module.meta.service.CentralMetaCryptoTypeService;
import com.loafle.overflow.core.model.PageParams;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.discovery.Host;
import com.loafle.overflow.model.discovery.Port;
import com.loafle.overflow.model.discovery.Zone;
import com.loafle.overflow.model.infra.Infra;
import com.loafle.overflow.model.infra.InfraHost;
import com.loafle.overflow.model.infra.InfraHostIP;
import com.loafle.overflow.model.infra.InfraHostOS;
import com.loafle.overflow.model.infra.InfraHostPort;
import com.loafle.overflow.model.infra.InfraZone;
import com.loafle.overflow.model.meta.MetaCryptoType;
import com.loafle.overflow.model.meta.MetaIPType;
import com.loafle.overflow.model.meta.MetaInfraType;
import com.loafle.overflow.model.meta.MetaPortType;
import com.loafle.overflow.model.meta.MetaTargetHostType;
import com.loafle.overflow.model.meta.MetaTargetZoneType;
import com.loafle.overflow.service.central.infra.InfraService;
import com.loafle.overflow.service.central.probe.ProbeService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import inet.ipaddr.IPAddress;
import inet.ipaddr.IPAddressString;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Service("InfraService")
public class CentralInfraService implements InfraService {
  @Autowired
  ProbeService probeService;

  @Autowired
  InfraDAO infraDAO;

  @Autowired
  InfraZoneDAO infraZoneDAO;

  @Autowired
  InfraHostDAO infraHostDAO;

  @Autowired
  InfraServiceDAO infraServiceDAO;

  @Autowired
  CentralInfraHostIPService infraHostIPService;

  @Autowired
  CentralInfraHostPortService infraHostPortService;

  @Autowired
  CentralMetaCryptoTypeService metaCryptoTypeService;

  @Override
  public Infra regist(Infra infra) throws OverflowException {
    return this.infraDAO.save(infra);
  }

  @Override
  @Transactional
  public InfraZone registByZone(Long probeID, Zone zone) throws OverflowException {
    if (null == zone) {
      throw new OverflowException("Zone is not valid");
    }

    Probe probe = this.probeService.read(probeID);
    if (null == probe) {
      throw new OverflowException(String.format("ID[%d] of Probe is not valid", probeID));
    }

    if (null == zone.getNetwork()) {
      throw new OverflowException("Network is not valid");
    }

    if (null == zone.getIface()) {
      throw new OverflowException("Iface is not valid");
    }

    InfraHostIP infraHostIP = new InfraHostIP();
    infraHostIP.setMetaIPType(zone.getMetaIPType());
    infraHostIP.setAddress(zone.getAddress());
    infraHostIP.setMac(zone.getMac());

    infraHostIP = infraHostIPService.validate(infraHostIP);

    InfraZone infraZone = this.infraZoneDAO.findByProbeIdAndNetwork(probeID, zone.getNetwork());
    if (null != infraZone) {
      return infraZone;
    }

    infraZone = new InfraZone();
    infraZone.setMetaInfraType(MetaInfraType.Enum.ZONE.to());
    infraZone.setProbe(probe);
    infraZone.setMetaTargetZoneType(MetaTargetZoneType.Enum.ZONE.to());
    infraZone.setNetwork(zone.getNetwork());
    infraZone.setIface(zone.getIface());
    infraZone.setMetaIPType(infraHostIP.getMetaIPType());
    infraZone.setAddress(infraHostIP.getAddress());
    infraZone.setMac(infraHostIP.getMac());

    return this.infraZoneDAO.save(infraZone);
  }

  @Override
  @Transactional
  public InfraHost registByHost(Long probeID, Host host) throws OverflowException {
    if (null == host) {
      throw new OverflowException("Host is not valid");
    }

    Probe probe = this.probeService.read(probeID);
    if (null == probe) {
      throw new OverflowException(String.format("ID[%d] of Probe is not valid", probeID));
    }

    Zone zone = host.getZone();
    if (null == zone) {
      throw new OverflowException("Zone is not valid");
    }

    InfraZone infraZone = this.registByZone(probeID, zone);

    InfraHostIP infraHostIP = new InfraHostIP();
    infraHostIP.setMetaIPType(host.getMetaIPType());
    infraHostIP.setAddress(host.getAddress());
    infraHostIP.setMac(host.getMac());

    infraHostIP = infraHostIPService.validate(infraHostIP);

    InfraHostIP eInfraHostIP = this.infraHostIPService.readByInfraHostInfraZoneIDAndMetaIPTypeKeyAndAddress(
        infraZone.getId(), infraHostIP.getMetaIPType().getKey(), host.getAddress());
    if (null != eInfraHostIP) {
      throw new OverflowException(String.format("IP[%s(%s)] of Zone[%s] is exist already", host.getAddress(),
          infraHostIP.getMetaIPType().getName(), infraZone.getNetwork()));
    }

    InfraHost infraHost = new InfraHost();
    infraZone.setMetaInfraType(MetaInfraType.Enum.HOST.to());
    infraZone.setProbe(probe);
    infraHost.setInfraZone(infraZone);
    infraHost.setMetaTargetHostType(MetaTargetHostType.Enum.UNKNOWN.to());
    infraHost.setInfraHostIPs(Arrays.asList(infraHostIP));

    return this.infraHostDAO.save(infraHost);
  }

  @Override
  @Transactional
  public com.loafle.overflow.model.infra.InfraService registByService(Long probeID,
      com.loafle.overflow.model.discovery.Service service) throws OverflowException {

    if (null == service) {
      throw new OverflowException("Service is not valid");
    }

    if (null == service.getMetaCryptoType()) {
      throw new OverflowException("MetaCryptoType is not valid");
    }

    MetaCryptoType metaCryptoType = this.metaCryptoTypeService.readByKey(service.getMetaCryptoType().getKey());
    if (null == metaCryptoType) {
      throw new OverflowException(
          String.format("MetaCryptoTypeKey[%s] is not valid", service.getMetaCryptoType().getKey()));
    }

    Probe probe = this.probeService.read(probeID);
    if (null == probe) {
      throw new OverflowException(String.format("ID[%d] of Probe is not valid", probeID));
    }

    Port port = service.getPort();
    if (null == port) {
      throw new OverflowException("Port of Service is not valid");
    }

    Host host = port.getHost();
    if (null == host) {
      throw new OverflowException("Host of Service is not valid");
    }

    Zone zone = host.getZone();
    if (null == zone) {
      throw new OverflowException("Zone is not valid");
    }

    InfraHostIP infraHostIP = new InfraHostIP();
    infraHostIP.setMetaIPType(host.getMetaIPType());
    infraHostIP.setAddress(host.getAddress());
    infraHostIP.setMac(host.getMac());

    infraHostIP = this.infraHostIPService.validate(infraHostIP);

    InfraHostPort infraHostPort = new InfraHostPort();
    infraHostPort.setMetaPortType(port.getMetaPortType());
    infraHostPort.setPort(port.getPortNumber());

    infraHostPort = this.infraHostPortService.validate(infraHostPort);

    InfraZone infraZone = this.registByZone(probeID, zone);
    InfraHost infraHost = this.registByHost(probeID, host);

    InfraHostIP eInfraHostIP = this.infraHostIPService.readByInfraHostIDAndMetaIPTypeKeyAndAddress(infraHost.getId(),
        infraHostIP.getMetaIPType().getKey(), infraHostIP.getAddress());
    if (null == eInfraHostIP) {
      throw new OverflowException(String.format("IP[%s(%s)] of Zone[%s] is not exist", host.getAddress(),
          infraHostIP.getMetaIPType().getName(), infraZone.getNetwork()));
    }
    infraHostIP = eInfraHostIP;

    InfraHostPort eInfraHostPort = this.infraHostPortService.readByInfraHostIDAndMetaPortTypeKeyAndPort(
        infraHost.getId(), infraHostPort.getMetaPortType().getKey(), infraHostPort.getPort());
    if (null != eInfraHostPort) {
      throw new OverflowException(String.format("Port[%d(%s)] of Zone[%s].Host[%s(%s)] is not exist", host.getAddress(),
          infraHostPort.getMetaPortType().getName(), infraZone.getNetwork(), infraHostIP.getAddress(),
          infraHostIP.getMetaIPType().getName()));
    }

    infraHostPort = this.infraHostPortService.regist(infraHostPort);

    com.loafle.overflow.model.infra.InfraService infraService = new com.loafle.overflow.model.infra.InfraService();
    infraService.setInfraHostIP(infraHostIP);
    infraService.setInfraHostPort(infraHostPort);
    infraService.setMetaCryptoType(metaCryptoType);

    return this.infraServiceDAO.save(infraService);
  }

  @Override
  @Transactional
  public List<Infra> registDiscoverd(Long probeID, List<Host> hosts,
      List<com.loafle.overflow.model.discovery.Service> services) throws OverflowException {

    List<Infra> infras = new ArrayList<>();

    if (null != hosts) {
      for (Host host : hosts) {
        infras.add(this.registByHost(probeID, host));
      }
    }
    if (null != services) {
      for (com.loafle.overflow.model.discovery.Service service : services) {
        infras.add(this.registByService(probeID, service));
      }
    }

    return infras;
  }

  public InfraZone registInfraZoneByInfraHostIP(Long probeID, InfraHostIP infraHostIP) throws OverflowException {
    if (null == infraHostIP) {
      throw new OverflowException("InfraHostIP is not valid");
    }

    infraHostIP = this.infraHostIPService.validate(infraHostIP);

    InfraZone infraZone = new InfraZone();
    infraZone.setProbe(new Probe(probeID));
    infraZone.setMetaInfraType(MetaInfraType.Enum.ZONE.to());
    infraZone.setMetaIPType(infraHostIP.getMetaIPType());
    infraZone.setAddress(infraHostIP.getAddress());
    infraZone.setIface(infraHostIP.getIface());
    infraZone.setMac(infraHostIP.getMac());

    IPAddress ipAddress = new IPAddressString(infraHostIP.getAddress()).getAddress();
    infraZone.setNetwork(ipAddress.toPrefixBlock().toString());

    return this.infraZoneDAO.save(infraZone);
  }

  public InfraHost registInfraHostByInfraHost(Long probeID, Long infraZoneID, InfraHost oriInfraHost)
      throws OverflowException {
    InfraHost infraHost = new InfraHost();
    infraHost.setProbe(new Probe(probeID));
    infraHost.setInfraZone(new InfraZone(infraZoneID));
    infraHost.setMetaInfraType(MetaInfraType.Enum.HOST.to());

    InfraHostOS infraHostOS = oriInfraHost.getInfraHostOS();
    if (null != infraHostOS) {
      // MetaTargetHostType metaTargetHostType =
      // this.metaTargetHostTypeService.readByInfraHostOS(infraHostOS);
      infraHost.setMetaTargetHostType(MetaTargetHostType.Enum.UNKNOWN.to());
      infraHost.setInfraHostOS(infraHostOS);
    } else {
      infraHost.setMetaTargetHostType(MetaTargetHostType.Enum.UNKNOWN.to());
    }

    infraHost.setInfraHostIPs(oriInfraHost.getInfraHostIPs());
    infraHost.setInfraHostPorts(oriInfraHost.getInfraHostPorts());
    infraHost.setInfraHostApplications(oriInfraHost.getInfraHostApplications());
    infraHost.setInfraHostDaemons(oriInfraHost.getInfraHostDaemons());

    return this.infraHostDAO.save(infraHost);
  }

  @Override
  public Infra read(Long id) throws OverflowException {
    return this.infraDAO.findById(id).get();
  }

  @Override
  public Page<Infra> readAllByProbeID(Long probeID, PageParams pageParams) throws OverflowException {
    return this.infraDAO.findAllByProbeId(probeID, PageUtil.getPageRequest(pageParams));
  }

  @Override
  public Page<Infra> readAllByDomainID(Long domainID, PageParams pageParams) throws OverflowException {
    return this.infraDAO.findAllByProbeDomainId(domainID, PageUtil.getPageRequest(pageParams));
  }

  @Override
  public List<Infra> readAllInfraByProbeID(Long probeID) throws OverflowException {
    return this.infraDAO.findAllByProbeId(probeID);
  }

  @Override
  public List<Infra> readAllInfraByDomainID(Long domainID) throws OverflowException {
    return this.infraDAO.findAllByProbeDomainId(domainID);
  }

  public List<InfraZone> readAllInfraZoneByProbeDomainID(Long probeDomainID) throws OverflowException {
    return null;
  }

}
