package com.loafle.overflow.central.module.target.service;

import java.util.List;

import com.loafle.overflow.central.commons.utils.PageUtil;
import com.loafle.overflow.central.module.probe.service.CentralProbeService;
import com.loafle.overflow.central.module.target.dao.TargetDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.core.model.PageParams;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.model.target.Target;
import com.loafle.overflow.service.central.target.TargetService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.transaction.annotation.Transactional;

/**
 * Created by insanity on 17. 6. 28.
 */
@org.springframework.stereotype.Service("TargetService")
public class CentralTargetService implements TargetService {

  @Autowired
  private TargetDAO targetDAO;
  @Autowired
  private CentralProbeService probeService;

  @Transactional
  public Target regist(Target target, Long probeID) throws OverflowException {
    this.probeService.increaseTargetCount(probeID);
    return this.targetDAO.save(target);
  }

  @Transactional
  public void remove(Long id, Long probeID) throws OverflowException {
    this.probeService.decreaseTargetCount(probeID);
    this.targetDAO.deleteById(id);
  }

  public Target read(Long id) throws OverflowException {
    return this.targetDAO.findById(id).get();
  }

  public Target modify(Target target) throws OverflowException {
    return this.targetDAO.save(target);
  }

  public Target increaseSensorCount(Long id) throws OverflowException {
    Target t = this.targetDAO.findById(id).get();
    t.setSensorCount(t.getSensorCount() + 1);
    return this.targetDAO.save(t);
  }

  public Target decreaseSensorCount(Long id) throws OverflowException {
    Target t = this.targetDAO.findById(id).get();
    int count = t.getSensorCount();
    if (t.getSensorCount() > 0) {
      t.setSensorCount(count - 1);
    }
    return this.targetDAO.save(t);
  }

  public Page<Target> readAllByProbeID(Long probeID, PageParams pageParams) throws OverflowException {
    return this.targetDAO.findAllByInfraProbeId(probeID, PageUtil.getPageRequest(pageParams));
  }

  @Transactional
  public List<Target> registAll(List<Target> targets, Long probeID) throws OverflowException {
    List<Target> registered = this.targetDAO.saveAll(targets);
    Probe probe = this.probeService.read(probeID);
    if (null == probe) {
      throw new OverflowException(String.format("ID[%d] of Probe is not valid", probeID));
    }
    Integer targetCount = probe.getTargetCount() + registered.size();
    probe.setTargetCount(targetCount);
    this.probeService.modify(probe);

    return registered;
  }

  public List<Target> readAllByZoneID() throws OverflowException {
    return null;
  }

  public List<Target> readAllTargetByProbeID(Long probeID) throws OverflowException {
    return null;
  }

}
