package com.loafle.overflow.central.spring;

import com.loafle.overflow.central.commons.service.MessagePublisher;
import com.loafle.overflow.central.redis.service.RedisMessagePublisher;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisStandaloneConfiguration;
import org.springframework.data.redis.connection.jedis.JedisClientConfiguration;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.connection.jedis.JedisClientConfiguration.JedisClientConfigurationBuilder;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.listener.ChannelTopic;
import org.springframework.data.redis.serializer.GenericToStringSerializer;

import java.time.Duration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Configuration
public class RedisConfiguration {
  @Value("${redis.host}")
  private String host;
  @Value("${redis.port}")
  private int port;
  @Value("#{'${redis.channels}'.split(',')}")
  private List<String> channels;

  @Bean
  JedisConnectionFactory jedisConnectionFactory() {
    RedisStandaloneConfiguration redisStandaloneConfiguration = new RedisStandaloneConfiguration();
    redisStandaloneConfiguration.setHostName(host);
    redisStandaloneConfiguration.setPort(port);
    // redisStandaloneConfiguration.setDatabase(0);
    // redisStandaloneConfiguration.setPassword(RedisPassword.of("password"));

    JedisClientConfigurationBuilder jedisClientConfiguration = JedisClientConfiguration.builder();
    jedisClientConfiguration.connectTimeout(Duration.ofSeconds(60));// 60s connection timeout

    JedisConnectionFactory jedisConFactory = new JedisConnectionFactory(redisStandaloneConfiguration,
        jedisClientConfiguration.build());

    // JedisConnectionFactory jedisConFactory = new JedisConnectionFactory();
    // jedisConFactory.setHostName(host);
    // jedisConFactory.setPort(port);
    return jedisConFactory;
  }

  @Bean
  public RedisTemplate<String, Object> redisTemplate() {
    final RedisTemplate<String, Object> template = new RedisTemplate<String, Object>();
    template.setConnectionFactory(jedisConnectionFactory());
    template.setValueSerializer(new GenericToStringSerializer<Object>(Object.class));
    return template;
  }

  @Bean
  MessagePublisher messagePublisher() {
    return new RedisMessagePublisher(redisTemplate(), topics());
  }

  @Bean
  Map<String, ChannelTopic> topics() {
    Map<String, ChannelTopic> topicMap = new HashMap<>(this.channels.size());
    for (String channel : this.channels) {
      topicMap.put(channel, new ChannelTopic(channel));
    }
    return topicMap;
  }

}
